"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var plugin_syntax_jsx_1 = __importDefault(require("@babel/plugin-syntax-jsx"));
var template_1 = __importDefault(require("@babel/template"));
var t = __importStar(require("@babel/types"));
var utils_1 = require("@compiled/utils");
var class_names_1 = require("./class-names");
var css_prop_1 = require("./css-prop");
var styled_1 = require("./styled");
var append_runtime_imports_1 = require("./utils/append-runtime-imports");
var cache_1 = require("./utils/cache");
var is_compiled_1 = require("./utils/is-compiled");
// eslint-disable-next-line @typescript-eslint/no-var-requires
var packageJson = require('../package.json');
var JSX_SOURCE_ANNOTATION_REGEX = /\*?\s*@jsxImportSource\s+([^\s]+)/;
var JSX_ANNOTATION_REGEX = /\*?\s*@jsx\s+([^\s]+)/;
var COMPILED_MODULE = '@compiled/react';
var globalCache;
exports.default = (0, helper_plugin_utils_1.declare)(function (api) {
    api.assertVersion(7);
    return {
        name: packageJson.name,
        inherits: plugin_syntax_jsx_1.default,
        pre: function () {
            this.sheets = {};
            var cache;
            if (this.opts.cache === true) {
                globalCache = new cache_1.Cache();
                cache = globalCache;
            }
            else {
                cache = new cache_1.Cache();
            }
            cache.initialize(__assign(__assign({}, this.opts), { cache: !!this.opts.cache }));
            this.cache = cache;
            this.includedFiles = [];
            this.pathsToCleanup = [];
            this.pragma = {};
        },
        visitor: {
            Program: {
                enter: function (_, state) {
                    var file = state.file;
                    if (file.ast.comments) {
                        for (var _i = 0, _a = file.ast.comments; _i < _a.length; _i++) {
                            var comment = _a[_i];
                            var jsxSourceMatches = JSX_SOURCE_ANNOTATION_REGEX.exec(comment.value);
                            var jsxMatches = JSX_ANNOTATION_REGEX.exec(comment.value);
                            if (jsxSourceMatches && jsxSourceMatches[1] === COMPILED_MODULE) {
                                // jsxImportSource pragma found - turn on CSS prop!
                                state.compiledImports = {};
                                state.pragma.jsxImportSource = true;
                            }
                            if (jsxMatches && jsxMatches[1] === 'jsx') {
                                state.pragma.jsx = true;
                            }
                        }
                    }
                },
                exit: function (path, state) {
                    var _a;
                    if (!state.compiledImports) {
                        return;
                    }
                    var pragma = state.pragma, _b = state.opts.importReact, shouldImportReact = _b === void 0 ? true : _b;
                    (0, utils_1.preserveLeadingComments)(path);
                    (0, append_runtime_imports_1.appendRuntimeImports)(path);
                    var hasPragma = pragma.jsxImportSource || pragma.jsx;
                    if (!hasPragma && shouldImportReact && !path.scope.getBinding('React')) {
                        // React is missing - add it in at the last moment!
                        path.unshiftContainer('body', template_1.default.ast("import * as React from 'react'"));
                    }
                    if (state.compiledImports.styled && !path.scope.getBinding('forwardRef')) {
                        // forwardRef is missing - add it in at the last moment!
                        path.unshiftContainer('body', template_1.default.ast("import { forwardRef } from 'react'"));
                    }
                    var filename = (0, path_1.basename)((_a = state.filename) !== null && _a !== void 0 ? _a : '') || 'File';
                    var version = process.env.TEST_PKG_VERSION || packageJson.version;
                    path.addComment('leading', " ".concat(filename, " generated by ").concat(packageJson.name, " v").concat(version, " "));
                    // Add a line break after the comment
                    path.unshiftContainer('body', t.noop());
                    // Callback when included files have been added.
                    if (this.includedFiles.length && this.opts.onIncludedFiles) {
                        this.opts.onIncludedFiles((0, utils_1.unique)(this.includedFiles));
                    }
                    // Cleanup paths that have been marked.
                    state.pathsToCleanup.forEach(function (clean) {
                        switch (clean.action) {
                            case 'remove': {
                                clean.path.remove();
                                return;
                            }
                            case 'replace': {
                                clean.path.replaceWith(t.nullLiteral());
                                return;
                            }
                            default:
                                return;
                        }
                    });
                },
            },
            ImportDeclaration: function (path, state) {
                if (path.node.source.value !== COMPILED_MODULE) {
                    return;
                }
                // The presence of the module enables CSS prop
                state.compiledImports = state.compiledImports || {};
                // Go through each import and enable each found API
                path.get('specifiers').forEach(function (specifier) {
                    if (!state.compiledImports || !specifier.isImportSpecifier()) {
                        // Bail out early
                        return;
                    }
                    ['styled', 'ClassNames', 'css', 'keyframes'].forEach(function (apiName) {
                        var _a, _b;
                        if (state.compiledImports &&
                            t.isIdentifier((_a = specifier.node) === null || _a === void 0 ? void 0 : _a.imported) &&
                            ((_b = specifier.node) === null || _b === void 0 ? void 0 : _b.imported.name) === apiName) {
                            // Enable the API with the local name
                            state.compiledImports[apiName] = specifier.node.local.name;
                            specifier.remove();
                        }
                    });
                });
                if (path.node.specifiers.length === 0) {
                    path.remove();
                }
            },
            TaggedTemplateExpression: function (path, state) {
                if ((0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledKeyframesTaggedTemplateExpression)(path.node, state)) {
                    state.pathsToCleanup.push({ path: path, action: 'replace' });
                    return;
                }
                if ((0, is_compiled_1.isCompiledStyledTaggedTemplateExpression)(path.node, state)) {
                    (0, styled_1.visitStyledPath)(path, { context: 'root', state: state, parentPath: path });
                    return;
                }
            },
            CallExpression: function (path, state) {
                if ((0, is_compiled_1.isCompiledCSSCallExpression)(path.node, state) ||
                    (0, is_compiled_1.isCompiledKeyframesCallExpression)(path.node, state)) {
                    state.pathsToCleanup.push({ path: path, action: 'replace' });
                    return;
                }
                if ((0, is_compiled_1.isCompiledStyledCallExpression)(path.node, state)) {
                    (0, styled_1.visitStyledPath)(path, { context: 'root', state: state, parentPath: path });
                    return;
                }
            },
            JSXElement: function (path, state) {
                var _a;
                if (!((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.ClassNames)) {
                    return;
                }
                (0, class_names_1.visitClassNamesPath)(path, { context: 'root', state: state, parentPath: path });
            },
            JSXOpeningElement: function (path, state) {
                if (!state.compiledImports) {
                    return;
                }
                (0, css_prop_1.visitCssPropPath)(path, { context: 'root', state: state, parentPath: path });
            },
        },
    };
});
//# sourceMappingURL=babel-plugin.js.map