"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildCompiledComponent = exports.compiledTemplate = void 0;
var generator_1 = __importDefault(require("@babel/generator"));
var template_1 = __importDefault(require("@babel/template"));
var t = __importStar(require("@babel/types"));
var utils_1 = require("@compiled/utils");
var build_css_variables_1 = require("./build-css-variables");
var get_jsx_attribute_1 = require("./get-jsx-attribute");
var hoist_sheet_1 = require("./hoist-sheet");
var transform_css_items_1 = require("./transform-css-items");
/**
 * Will return a generated AST for a Compiled Component.
 * This is primarily used for CSS prop and ClassNames apis.
 *
 * @param node Originating node
 * @param sheets {string[]} Stylesheets
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var compiledTemplate = function (node, sheets, meta) {
    var nonceAttribute = meta.state.opts.nonce ? "nonce={".concat(meta.state.opts.nonce, "}") : '';
    var keyAttribute = (0, get_jsx_attribute_1.getJSXAttribute)(node, 'key')[0];
    return (0, template_1.default)("\n  <CC ".concat(keyAttribute ? (0, generator_1.default)(keyAttribute).code : '', ">\n    <CS ").concat(nonceAttribute, ">{%%cssNode%%}</CS>\n    {%%jsxNode%%}\n  </CC>\n  "), {
        plugins: ['jsx'],
    })({
        jsxNode: node,
        cssNode: t.arrayExpression((0, utils_1.unique)(sheets).map(function (sheet) { return (0, hoist_sheet_1.hoistSheet)(sheet, meta); })),
    });
};
exports.compiledTemplate = compiledTemplate;
/**
 * Returns the actual value of a jsx value.
 *
 * @param node
 */
var getExpression = function (node) {
    var value = t.isJSXExpressionContainer(node) ? node.expression : node;
    if (t.isJSXEmptyExpression(value)) {
        throw new Error('Empty expression not supported.');
    }
    return value;
};
/**
 * Returns a Compiled Component AST.
 *
 * @param node Originating node
 * @param cssOutput CSS and variables to place onto the component
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var buildCompiledComponent = function (node, cssOutput, meta) {
    var _a = (0, transform_css_items_1.transformCssItems)(cssOutput.css), sheets = _a.sheets, classNames = _a.classNames;
    var classNameAttribute = (0, get_jsx_attribute_1.getJSXAttribute)(node, 'className')[0];
    if (classNameAttribute && classNameAttribute.value) {
        // If there is a class name prop statically defined we want to concatenate it with
        // the class name we're going to put on it.
        var classNameExpression = getExpression(classNameAttribute.value);
        var values = classNames.concat(classNameExpression);
        classNameAttribute.value = t.jsxExpressionContainer(t.callExpression(t.identifier('ax'), [t.arrayExpression(values)]));
    }
    else {
        // No class name - just push our own one.
        node.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('className'), t.jsxExpressionContainer(t.callExpression(t.identifier('ax'), [t.arrayExpression(classNames)]))));
    }
    if (cssOutput.variables.length) {
        var _b = (0, get_jsx_attribute_1.getJSXAttribute)(node, 'style'), styleAttribute = _b[0], styleAttributeIndex = _b[1];
        var dynamicStyleProperties_1 = (0, build_css_variables_1.buildCssVariables)(cssOutput.variables);
        if (styleAttribute) {
            // Remove the pre-existing style prop - we're going to redefine it soon.
            node.openingElement.attributes.splice(styleAttributeIndex, 1);
            if (styleAttribute.value &&
                t.isJSXExpressionContainer(styleAttribute.value) &&
                !t.isJSXEmptyExpression(styleAttribute.value.expression)) {
                // If it's not an object we just spread the expression into the object
                if (!t.isObjectExpression(styleAttribute.value.expression)) {
                    dynamicStyleProperties_1.splice(0, 0, t.spreadElement(styleAttribute.value.expression));
                }
                else {
                    // Else it's an object! So we want to place each property into the object
                    styleAttribute.value.expression.properties.forEach(function (prop, index) {
                        if (t.isObjectMethod(prop)) {
                            return;
                        }
                        // We want to keep the order that they were defined in.
                        // So we're using index here to do just that.
                        dynamicStyleProperties_1.splice(index, 0, prop);
                    });
                }
            }
        }
        // Finally add the new style prop back to the opening JSX element.
        node.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('style'), t.jsxExpressionContainer(t.objectExpression(dynamicStyleProperties_1))));
    }
    return (0, exports.compiledTemplate)(node, sheets, meta);
};
exports.buildCompiledComponent = buildCompiledComponent;
//# sourceMappingURL=build-compiled-component.js.map