"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildStyledComponent = void 0;
var generator_1 = __importDefault(require("@babel/generator"));
var template_1 = __importDefault(require("@babel/template"));
var traverse_1 = __importDefault(require("@babel/traverse"));
var t = __importStar(require("@babel/types"));
var css_1 = require("@compiled/css");
var utils_1 = require("@compiled/utils");
var is_prop_valid_1 = __importDefault(require("@emotion/is-prop-valid"));
var constants_1 = require("../constants");
var ast_1 = require("./ast");
var build_css_variables_1 = require("./build-css-variables");
var css_builders_1 = require("./css-builders");
var hoist_sheet_1 = require("./hoist-sheet");
var transform_css_items_1 = require("./transform-css-items");
/**
 * Builds up the inline style prop value for a Styled Component.
 *
 * @param variables CSS variables that will be placed in the AST
 * @param transform Transform callback function that can be used to change the CSS variable expression
 */
var styledStyleProp = function (variables, transform) {
    var props = [t.spreadElement(t.identifier('style'))];
    return t.objectExpression(props.concat((0, build_css_variables_1.buildCssVariables)(variables, transform)));
};
/**
 * Returns a tag string in the form of an identifier or string literal.
 *
 * A type of InBuiltComponent will return a string literal,
 * otherwise an identifier string will be returned.
 *
 * @param tag Made of name and type.
 */
var buildComponentTag = function (_a) {
    var name = _a.name, type = _a.type;
    return type === 'InBuiltComponent' ? "\"".concat(name, "\"") : name;
};
/**
 * Traverses an arrow function and then finally return the arrow function body node.
 *
 * @param node Array function node
 * @param nestedVisitor Visitor callback function
 */
var traverseStyledArrowFunctionExpression = function (node, nestedVisitor) {
    (0, traverse_1.default)(node, nestedVisitor);
    return (0, ast_1.pickFunctionBody)(node);
};
/**
 * Traverses a binary expression looking for any arrow functions,
 * calls back with each arrow function node into the passed in `nestedVisitor`,
 * and then finally replaces each found arrow function node with its body.
 *
 * @param node Binary expression node
 * @param nestedVisitor Visitor callback function
 */
var traverseStyledBinaryExpression = function (node, nestedVisitor) {
    (0, traverse_1.default)(node, {
        noScope: true,
        ArrowFunctionExpression: function (path) {
            path.traverse(nestedVisitor);
            path.replaceWith((0, ast_1.pickFunctionBody)(path.node));
            path.stop();
        },
    });
    return node;
};
/**
 * Returns a list of destructured props used in expressions of a styled component
 *
 * For example:
 * ```
 * const Component = styled.div`
 *  width: $({ width }) => `${width}px`;
 *  height: $({ height }) => `${height}px`;
 * `;
 * ```
 * Returns list of [width, height]
 *
 * @param node {t.Node} Node of the styled component
 */
var getDestructuredProps = function (node) {
    var destructuredProps = [];
    (0, traverse_1.default)(node, {
        noScope: true,
        ArrowFunctionExpression: function (path) {
            var propsParam = path.get('params')[0];
            // Is the param for props being destructured
            if (propsParam && t.isObjectPattern(propsParam.node)) {
                // Uses the destructure ObjectPattern path to get all the prop references, i.e.
                // { width } becomes width
                // { width, height } becomes width,height
                // { size: { width } } becomes size:{width}
                // { width: alias } becomes width:alias
                var propsUsed = propsParam.toString().replace(/\s/g, '').slice(1, -1).split(',');
                destructuredProps.push.apply(destructuredProps, propsUsed);
            }
        },
    });
    return destructuredProps;
};
/**
 * Handles cases like:
 * 1. `propz.loading` in `border-color: \${(propz) => (propz.loading ? colors.N100 : colors.N200)};`
 * Outcome: It will replace `propz.loading` with `props.loading`.
 *
 * 2. `props.notValidProp` in `border-color: \${(props) => (props.notValidProp ? colors.N100 : colors.N200)};`
 * Outcome: It will move `notValidProp` under `propsToDestructure` and replaces `props.notValidProp` with `notValidProp`.
 *
 * @param path MemberExpression path
 */
var handleMemberExpressionInStyledInterpolation = function (path) {
    var memberExpressionKey = path.node.object;
    var propsToDestructure = [];
    if (t.isIdentifier(memberExpressionKey)) {
        var traversedUpFunctionPath = path.find(function (parentPath) {
            return parentPath.isFunction();
        });
        var memberExpressionKeyName = memberExpressionKey.name;
        var isMemberExpressionNameTheSameAsFunctionFirstParam = traversedUpFunctionPath &&
            t.isFunction(traversedUpFunctionPath.node) &&
            t.isIdentifier(traversedUpFunctionPath.node.params[0]) &&
            traversedUpFunctionPath.node.params[0].name === memberExpressionKeyName;
        if (isMemberExpressionNameTheSameAsFunctionFirstParam) {
            var memberExpressionValue = path.node.property;
            if (t.isIdentifier(memberExpressionValue)) {
                var memberExpressionValueName = memberExpressionValue.name;
                // if valid html attribute let it through - else destructure to prevent
                if ((0, is_prop_valid_1.default)(memberExpressionValueName)) {
                    // Convert cases like propz.color to props.color
                    if (memberExpressionKeyName !== constants_1.PROPS_IDENTIFIER_NAME) {
                        path.replaceWith(t.memberExpression(t.identifier(constants_1.PROPS_IDENTIFIER_NAME), t.identifier(memberExpressionValueName)));
                    }
                }
                else {
                    propsToDestructure.push(memberExpressionValueName);
                    path.replaceWith(memberExpressionValue);
                }
            }
        }
    }
    return propsToDestructure;
};
/**
 * Will return a generated AST for a Styled Component.
 *
 * @param opts {StyledTemplateOpts} Template options
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var styledTemplate = function (opts, meta) {
    var nonceAttribute = meta.state.opts.nonce ? "nonce={".concat(meta.state.opts.nonce, "}") : '';
    // This completely depends on meta.parentPath.node to be the styled component.
    // If this changes please pass the component in another way
    var propsToDestructure = getDestructuredProps(meta.parentPath.node);
    var styleProp = opts.variables.length
        ? styledStyleProp(opts.variables, function (node) {
            var nestedArrowFunctionExpressionVisitor = {
                noScope: true,
                MemberExpression: function (path) {
                    var propsToDestructureFromMemberExpression = handleMemberExpressionInStyledInterpolation(path);
                    propsToDestructure.push.apply(propsToDestructure, propsToDestructureFromMemberExpression);
                },
            };
            if (t.isArrowFunctionExpression(node)) {
                return traverseStyledArrowFunctionExpression(node, nestedArrowFunctionExpressionVisitor);
            }
            if (t.isBinaryExpression(node)) {
                return traverseStyledBinaryExpression(node, nestedArrowFunctionExpressionVisitor);
            }
            return node;
        })
        : t.identifier('style');
    var unconditionalClassNames = '', conditionalClassNames = '';
    opts.classNames.forEach(function (item) {
        if (t.isStringLiteral(item)) {
            unconditionalClassNames += "".concat(item.value, " ");
        }
        else if (t.isLogicalExpression(item) || t.isConditionalExpression(item)) {
            conditionalClassNames += "".concat((0, generator_1.default)(item).code, ", ");
        }
    });
    var classNames = "\"".concat(unconditionalClassNames.trim(), "\", ").concat(conditionalClassNames);
    return (0, template_1.default)("\n  forwardRef(({\n    as: C = ".concat(buildComponentTag(opts.tag), ",\n    style,\n    ").concat((0, utils_1.unique)(propsToDestructure)
        .map(function (prop) { return prop + ','; })
        .join(''), "\n    ...").concat(constants_1.PROPS_IDENTIFIER_NAME, "\n  }, ref) => (\n    <CC>\n      <CS ").concat(nonceAttribute, ">{%%cssNode%%}</CS>\n      <C\n        {...").concat(constants_1.PROPS_IDENTIFIER_NAME, "}\n        style={%%styleProp%%}\n        ref={ref}\n        className={ax([").concat(classNames, " ").concat(constants_1.PROPS_IDENTIFIER_NAME, ".className])}\n      />\n    </CC>\n  ));\n"), {
        plugins: ['jsx'],
    })({
        styleProp: styleProp,
        cssNode: t.arrayExpression((0, utils_1.unique)(opts.sheets).map(function (sheet) { return (0, hoist_sheet_1.hoistSheet)(sheet, meta); })),
    });
};
/**
 * Find CSS selectors that are apart of incomplete closures
 * i.e. `:hover {`
 *
 * @param css {string} Template options
 */
var findOpenSelectors = function (css) {
    // Remove any occurrence of { or } inside quotes to stop them
    // interfering with closure matches
    var searchArea = css.replace(/['|"].*[{|}].*['|"]/g, '');
    // Skip over complete closures
    searchArea = searchArea.substring(searchArea.lastIndexOf('}') + 1);
    // Regex for CSS selector
    //[^;\s] Don't match ; or whitespace
    // .+\n?{ Match anything (the selector itself) followed by any newlines then {
    return searchArea.match(/[^;\s].+\n?{/g);
};
/**
 * Returns a Styled Component AST.
 *
 * @param tag {Tag} Styled tag either an inbuilt or user define
 * @param cssOutput {CSSOutput} CSS and variables to place onto the component
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var buildStyledComponent = function (tag, cssOutput, meta) {
    var unconditionalCss = '';
    var conditionalCssItems = [];
    cssOutput.css.forEach(function (item) {
        if (item.type === 'logical' || item.type === 'conditional') {
            // TODO: Optimize this to only run if there is a
            // potential selector scope change
            var selectors = findOpenSelectors(unconditionalCss);
            if (selectors) {
                (0, transform_css_items_1.applySelectors)(item, selectors);
            }
            conditionalCssItems.push(item);
        }
        else {
            unconditionalCss += (0, css_builders_1.getItemCss)(item);
        }
    });
    // Rely on transformCss to remove duplicates and return only the last unconditional CSS for each property
    var uniqueUnconditionalCssOutput = (0, css_1.transformCss)(unconditionalCss);
    // Rely on transformItemCss to build expressions for conditional & logical CSS
    var conditionalCssOutput = (0, transform_css_items_1.transformCssItems)(conditionalCssItems);
    var sheets = __spreadArray(__spreadArray([], uniqueUnconditionalCssOutput.sheets, true), conditionalCssOutput.sheets, true);
    var classNames = __spreadArray(__spreadArray([], [t.stringLiteral(uniqueUnconditionalCssOutput.classNames.join(' '))], false), conditionalCssOutput.classNames, true);
    return styledTemplate({
        classNames: classNames,
        tag: tag,
        sheets: sheets,
        variables: cssOutput.variables,
    }, meta);
};
exports.buildStyledComponent = buildStyledComponent;
//# sourceMappingURL=build-styled-component.js.map