"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isCompiledStyledTaggedTemplateExpression = exports.isCompiledStyledCallExpression = exports.isCompiledKeyframesTaggedTemplateExpression = exports.isCompiledKeyframesCallExpression = exports.isCompiledCSSTaggedTemplateExpression = exports.isCompiledCSSCallExpression = void 0;
var t = __importStar(require("@babel/types"));
/**
 * Returns `true` if the node is using `css` from `@compiled/react` as a call expression
 *
 * @param node {t.Expression} The expression node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a css usage from compiled
 */
var isCompiledCSSCallExpression = function (node, state) {
    var _a;
    return t.isCallExpression(node) &&
        t.isIdentifier(node.callee) &&
        node.callee.name === ((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.css);
};
exports.isCompiledCSSCallExpression = isCompiledCSSCallExpression;
/**
 * Returns `true` if the node is using `css` from `@compiled/react` as a tagged template expression
 *
 * @param node {t.Expression} The expression node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a css usage from compiled
 */
var isCompiledCSSTaggedTemplateExpression = function (node, state) {
    var _a;
    return t.isTaggedTemplateExpression(node) &&
        t.isIdentifier(node.tag) &&
        node.tag.name === ((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.css);
};
exports.isCompiledCSSTaggedTemplateExpression = isCompiledCSSTaggedTemplateExpression;
/**
 * Returns `true` if the node is using `keyframes` from `@compiled/react` as a call expression
 *
 * @param node {t.Node} The node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a compiled keyframe
 */
var isCompiledKeyframesCallExpression = function (node, state) {
    var _a;
    return t.isCallExpression(node) &&
        t.isIdentifier(node.callee) &&
        node.callee.name === ((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.keyframes);
};
exports.isCompiledKeyframesCallExpression = isCompiledKeyframesCallExpression;
/**
 * Returns `true` if the node is using `keyframes` from `@compiled/react` as a tagged template expression
 *
 * @param node {t.Node} The node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a compiled keyframe
 */
var isCompiledKeyframesTaggedTemplateExpression = function (node, state) {
    var _a;
    return t.isTaggedTemplateExpression(node) &&
        t.isIdentifier(node.tag) &&
        node.tag.name === ((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.keyframes);
};
exports.isCompiledKeyframesTaggedTemplateExpression = isCompiledKeyframesTaggedTemplateExpression;
/**
 * Returns `true` if the node is using `styled` from `@compiled/react` from a styled.tag usage
 *
 * @param node {t.Node} The node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a styled usage from compiled
 */
var isCompiledStyledMemberExpression = function (node, state) {
    var _a;
    return t.isMemberExpression(node) &&
        t.isIdentifier(node.object) &&
        node.object.name === ((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.styled);
};
/**
 * Returns `true` if the node is using `styled` from `@compiled/react` from a styled(Component) usage
 *
 * @param node {t.Node} The node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a styled usage from compiled
 */
var isCompiledStyledCompositionCallExpression = function (node, state) {
    var _a;
    return t.isCallExpression(node) &&
        t.isIdentifier(node.callee) &&
        node.callee.name === ((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.styled);
};
/**
 * Returns `true` if the node is using `styled` from `@compiled/react` as a call expression
 *
 * @param node {t.Expression} The expression node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a styled usage from compiled
 */
var isCompiledStyledCallExpression = function (node, state) {
    return t.isCallExpression(node) &&
        (isCompiledStyledMemberExpression(node.callee, state) ||
            isCompiledStyledCompositionCallExpression(node.callee, state));
};
exports.isCompiledStyledCallExpression = isCompiledStyledCallExpression;
/**
 * Returns `true` if the node is using `styled` from `@compiled/react` as a tagged template expression
 *
 * @param node {t.Node} The node that is being checked
 * @param state {State} Plugin state
 * @returns {boolean} Whether the node is a styled usage from compiled
 */
var isCompiledStyledTaggedTemplateExpression = function (node, state) {
    return t.isTaggedTemplateExpression(node) &&
        (isCompiledStyledMemberExpression(node.tag, state) ||
            isCompiledStyledCompositionCallExpression(node.tag, state));
};
exports.isCompiledStyledTaggedTemplateExpression = isCompiledStyledTaggedTemplateExpression;
//# sourceMappingURL=is-compiled.js.map