"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.traverseIdentifier = void 0;
var create_result_pair_1 = require("../create-result-pair");
var resolve_binding_1 = require("../resolve-binding");
/**
 * Will look in an expression and return the actual value along with updated metadata.
 *
 * E.g: If there is an identifier called `color` that is set somewhere as `const color = 'blue'`,
 * passing the `color` identifier to this function would return `'blue'`.
 *
 * @param expression Expression we want to interrogate.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var traverseIdentifier = function (expression, meta, evaluateExpression) {
    var _a;
    var value = undefined;
    var updatedMeta = meta;
    var resolvedBinding = (0, resolve_binding_1.resolveBinding)(expression.name, updatedMeta, evaluateExpression);
    if (resolvedBinding && resolvedBinding.constant && resolvedBinding.node) {
        // We recursively call get interpolation until it not longer returns an identifier or member expression
        (_a = evaluateExpression(resolvedBinding.node, resolvedBinding.meta), value = _a.value, updatedMeta = _a.meta);
    }
    return (0, create_result_pair_1.createResultPair)(value, updatedMeta);
};
exports.traverseIdentifier = traverseIdentifier;
//# sourceMappingURL=traverse-identifier.js.map